/*$Id: lang_spice_commands.cc,v 26.52 2007/11/02 21:15:21 al Exp $ -*- C++ -*-
 * Copyright (C) 2006 Albert Davis
 * Author: Albert Davis <aldavis@ieee.org>
 *
 * This file is part of "Gnucap", the Gnu Circuit Analysis Package
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */
#include "d_dot.h"
#include "d_coment.h"
#include "d_subckt.h"
#include "e_model.h"
#include "c_comand.h"
#include "l_dispatcher.h"
#include "lang_spice.h"
extern DISPATCHER<CMD> command_dispatcher;
extern DISPATCHER<LANGUAGE> language_dispatcher;
extern DISPATCHER<MODEL_CARD> model_dispatcher;
extern DISPATCHER<CARD> device_dispatcher;
extern std::string head;	    /* place to store title line            */
/*--------------------------------------------------------------------------*/
namespace {
/*--------------------------------------------------------------------------*/
LANG_SPICE lang_spice;
DISPATCHER<LANGUAGE>::INSTALL
	d(&language_dispatcher, lang_spice.name(), &lang_spice);
DEV_COMMENT p0;
DISPATCHER<CARD>::INSTALL
	d0(&device_dispatcher, ";,#,*,',\",dev_comment", &p0);
/*--------------------------------------------------------------------------*/
class CMD_MODEL : public CMD {
public:
  void do_it(CS& cmd, CARD_LIST* scope)
  {
    // already got "model"
    std::string my_name, base_name;
    cmd >> my_name;
    int here = cmd.cursor();    
    cmd >> base_name;
    
    // "level" kluge ....
    // if there is a "level" keyword, with integer argument,
    // tack that onto the given modelname and look for that
    cmd.skip1b('(');
    int level = 0;
    {
      int here = cmd.cursor();
      scan_get(cmd, "LEVEL", &level);
      if (!cmd.stuck(&here)) {
	char buf[20];
	sprintf(buf, "%u", level);
	base_name += buf;
      }else{
      }
    }

    const MODEL_CARD* p = model_dispatcher[base_name];

    // case kluge.  Try exact, then tolower then toupper.
    if (!p) {
      notstd::to_lower(&base_name);
      p = model_dispatcher[base_name];
    }else{
    }
    if (!p) {untested();
      notstd::to_upper(&base_name);
      p = model_dispatcher[base_name];
    }else{
    }
    if (p) {
      MODEL_CARD* new_card = dynamic_cast<MODEL_CARD*>(p->clone());
      if (exists(new_card)) {
	assert(!new_card->owner());
	lang_spice.parse_paramset(cmd, new_card);
	scope->push_back(new_card);
      }else{untested();
	cmd.warn(bDANGER, here, "model: base has incorrect type");
      }
    }else{untested();
      cmd.warn(bDANGER, here, "model: no match");
    }
  }
} p1;
DISPATCHER<CMD>::INSTALL d1(&command_dispatcher, ".model", &p1);
/*--------------------------------------------------------------------------*/
class CMD_SUBCKT : public CMD {
public:
  void do_it(CS& cmd, CARD_LIST* scope)
  {untested();
    MODEL_SUBCKT* new_module = new MODEL_SUBCKT;
    assert(new_module);
    assert(!new_module->owner());
    assert(new_module->subckt());
    assert(new_module->subckt()->is_empty());
    lang_spice.parse_module(cmd, new_module);
    scope->push_back(new_module);
  }
} p2;
DISPATCHER<CMD>::INSTALL d2(&command_dispatcher, ".subckt,.macro", &p2);
/*--------------------------------------------------------------------------*/
enum Skip_Header {NO_HEADER, SKIP_HEADER};
/*--------------------------------------------------------------------------*/
/* getmerge: actually do the work for "get", "merge", etc.
 */
static void getmerge(CS& cmd, Skip_Header skip_header, CARD_LIST* scope)
{
  assert(scope);

  SIM::uninit();

  std::string file_name;
  cmd >> file_name;
  
  bool  echoon = false;		/* echo on/off flag (echo as read from file) */
  bool  liston = false;		/* list on/off flag (list actual values) */
  bool  quiet = false;		/* don't echo title */
  int here = cmd.cursor();
  do{
    ONE_OF
      || get(cmd, "Echo",  &echoon)
      || get(cmd, "List",  &liston)
      || get(cmd, "Quiet", &quiet)
      ;
  }while (cmd.more() && !cmd.stuck(&here));
  cmd.check(bWARNING, "need echo, list, or quiet");

  CS file(CS::_INC_FILE, file_name);

  if (skip_header) { // get and store the header line
    file.get_line(">>>>");
    head = file.fullstring();

    if (!quiet) {
      IO::mstdout << head << '\n';
    }else{untested();
    }
  }else{
  }

  lang_spice.parse_module_body(file, NULL, scope, ">>>>", lang_spice.NO_EXIT_ON_BLANK);
}
/*--------------------------------------------------------------------------*/
/* cmd_include: include command
 * as get or run, but do not clear first, inherit the run-mode.
 */
class CMD_INCLUDE : public CMD {
public:
  void do_it(CS& cmd, CARD_LIST* scope)
  {
    ::status.get.reset().start();
    getmerge(cmd, NO_HEADER, scope);
    ::status.get.stop();
  }
} p3;
DISPATCHER<CMD>::INSTALL d3(&command_dispatcher, "include", &p3);
/*--------------------------------------------------------------------------*/
/* cmd_merge: merge command
 * as get, but do not clear first
 */
class CMD_MERGE : public CMD {
public:
  void do_it(CS& cmd, CARD_LIST* scope)
  {untested();
    SET_RUN_MODE xx(rPRESET);
    ::status.get.reset().start();
    getmerge(cmd, NO_HEADER, scope);
    ::status.get.stop();
  }
} p4;
DISPATCHER<CMD>::INSTALL d4(&command_dispatcher, "merge", &p4);
/*--------------------------------------------------------------------------*/
/* cmd_run: "<" and "<<" commands
 * run in batch mode.  Spice format.
 * "<<" clears old circuit first, "<" does not
 * get circuit from file, execute dot cards in sequence
 */
class CMD_RUN : public CMD {
public:
  void do_it(CS& cmd, CARD_LIST* scope)
  {
    ::status.get.reset().start();
    while (cmd.match1('<')) {untested();
      command("clear", scope);
      cmd.skip();
      cmd.skipbl();
    }
    SET_RUN_MODE xx(rSCRIPT);
    getmerge(cmd, SKIP_HEADER, scope);
    ::status.get.stop();
  }
} p5;
DISPATCHER<CMD>::INSTALL d5(&command_dispatcher, "<", &p5);
/*--------------------------------------------------------------------------*/
/* cmd_get: get command
 * get circuit from a file, after clearing the old one
 * preset, but do not execute "dot cards"
 */
class CMD_GET : public CMD {
public:
  void do_it(CS& cmd, CARD_LIST* scope)
  {
    SET_RUN_MODE xx(rPRESET);
    ::status.get.reset().start();
    command("clear", scope);
    getmerge(cmd, SKIP_HEADER, scope);
    ::status.get.stop();
  }
} p6;
DISPATCHER<CMD>::INSTALL d6(&command_dispatcher, "get", &p6);
/*--------------------------------------------------------------------------*/
/* cmd_build: build command
 * get circuit description direct from keyboard (or stdin if redirected)
 * Command syntax: build <before>
 * Bare command: add to end of list
 * If there is an arg: add before that element
 * null line exits this mode
 * preset, but do not execute "dot cards"
 */
class CMD_BUILD : public CMD {
public:
  void do_it(CS& cmd, CARD_LIST* scope)
  {
    SET_RUN_MODE xx(rPRESET);
    ::status.get.reset().start();
    SIM::uninit();
    lang_spice.parse_module_body(cmd, NULL, scope, ">", lang_spice.EXIT_ON_BLANK);
    ::status.get.stop();
  }
} p7;
DISPATCHER<CMD>::INSTALL d7(&command_dispatcher, "build", &p7);
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
