/*$Id: u_lang.cc,v 26.52 2007/11/02 21:15:21 al Exp $ -*- C++ -*-
 * Copyright (C) 2006 Albert Davis
 * Author: Albert Davis <aldavis@ieee.org>
 *
 * This file is part of "Gnucap", the Gnu Circuit Analysis Package
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */
#include "d_dot.h"
#include "d_coment.h"
#include "d_subckt.h"
#include "e_model.h"
#include "l_dispatcher.h"
#include "u_lang.h"
extern DISPATCHER<LANGUAGE> language_dispatcher;
extern DISPATCHER<CARD> device_dispatcher;
/*--------------------------------------------------------------------------*/
const CARD* LANGUAGE::find_proto(const std::string& name, const CARD* scope)
{
  const CARD* p = NULL;
  if (scope) {
    p = scope->find_looking_out(name, bNOERROR);
  }else{
    CARD_LIST::const_iterator i = CARD_LIST::card_list.find_(name);
    if (i != CARD_LIST::card_list.end()) {untested();
      p = *i;
    }else{
      assert(!p);
    }
  }
  if (!p) {
    return device_dispatcher[name];
  }else if (const MODEL_CARD* m = dynamic_cast<const MODEL_CARD*>(p)) {
    return m->component_proto(); // type is COMPONENT*
  }else if (dynamic_cast<const MODEL_SUBCKT*>(p)) {itested();
    return device_dispatcher["dev_subckt"];
  }else{untested();
    return p;
  }
}
/*--------------------------------------------------------------------------*/
void LANGUAGE::new__instance(CS& cmd, MODEL_SUBCKT* owner)
{
  int here = cmd.cursor();
  std::string type;
  cmd >> type;
  const CARD* proto = find_proto(type, owner);
  if (proto) {
    CARD* new_instance = proto->clone();
    assert(new_instance);
    new_instance->set_owner(owner);
    parse_item(cmd, new_instance);
    owner->subckt()->push_back(new_instance);
  }else{untested();
    cmd.warn(bDANGER, here, "no match");
  }
}
/*--------------------------------------------------------------------------*/
void LANGUAGE::parse_item(CS& cmd, CARD* c)
{
  // See Stroustrup 15.4.5
  // If you can think of a better way, tell me.
  // It must be in the LANGUAGE class, not CARD.

  if (dynamic_cast<MODEL_SUBCKT*>(c)) {
    parse_module(cmd, prechecked_cast<MODEL_SUBCKT*>(c));
  }else if (dynamic_cast<COMPONENT*>(c)) {
    parse_instance(cmd, prechecked_cast<COMPONENT*>(c));
  }else if (dynamic_cast<MODEL_CARD*>(c)) {
    parse_paramset(cmd, prechecked_cast<MODEL_CARD*>(c));
  }else if (dynamic_cast< DEV_COMMENT*>(c)) {
    parse_comment(cmd, prechecked_cast<DEV_COMMENT*>(c));
  }else if (dynamic_cast<DEV_DOT*>(c)) {
    parse_command(cmd, prechecked_cast<DEV_DOT*>(c));
  }else{untested();
    incomplete();
    unreachable();
  }
}
/*--------------------------------------------------------------------------*/
void LANGUAGE::print_item(OMSTREAM& o, const CARD* c)
{
  // See Stroustrup 15.4.5
  // If you can think of a better way, tell me.
  // It must be in the LANGUAGE class, not CARD.

  if (dynamic_cast<const MODEL_SUBCKT*>(c)) {
    print_module(o, prechecked_cast<const MODEL_SUBCKT*>(c));
  }else if (dynamic_cast<const COMPONENT*>(c)) {
    print_instance(o, prechecked_cast<const COMPONENT*>(c));
  }else if (dynamic_cast<const MODEL_CARD*>(c)) {
    print_paramset(o, prechecked_cast<const MODEL_CARD*>(c));
  }else if (dynamic_cast<const DEV_COMMENT*>(c)) {
    print_comment(o, prechecked_cast<const DEV_COMMENT*>(c));
  }else if (dynamic_cast<const DEV_DOT*>(c)) {
    print_command(o, prechecked_cast<const DEV_DOT*>(c));
  }else{untested();
    incomplete();
    unreachable();
  }
}
/*--------------------------------------------------------------------------*/
OMSTREAM& operator<<(OMSTREAM& o, LANGUAGE* x)
{
  assert(x);
  return (o << x->name());
}
/*--------------------------------------------------------------------------*/
bool get(CS& cmd, const std::string& key, LANGUAGE** val)
{
  if (cmd.dmatch(key)) {
    cmd.skip1b('=');
    LANGUAGE* lang = language_dispatcher[cmd];
    if (lang) {
      *val = lang;
    }else{
    }
    return true;
  }else{
    return false;
  }
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
